"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const egg_aop_1 = require("egg-aop");
const egg_typed_tracking_1 = require("egg-typed-tracking");
const loader_util_1 = require("./loader_util");
const meta_1 = require("./meta/meta");
const tracking_1 = require("./middleware/tracking");
const _1 = require(".");
const fork_1 = require("./fork");
const EggLoader = require('egg').AppWorkerLoader;
exports.EggLoader = EggLoader;
function getSymbol(obj, symbol) {
    const desc = Object.getOwnPropertyDescriptor(obj, symbol);
    return desc && desc.value;
}
class AppWorkerLoader extends EggLoader {
    constructor() {
        super(...arguments);
        this.metadataPath = path.join(this.baseDir, 'run');
        this.registeredTS = false;
    }
    get etConfig() {
        return ((this.app.config && this.app.config.et) || {
            useTSRuntime: false,
        });
    }
    get baseDir() {
        return this.options.baseDir;
    }
    /**
     * 开始加载所有约定目录
     * @since 1.0.0
     */
    load() {
        if (this.etConfig.useTSRuntime) {
            loader_util_1.registerTSNode(this.baseDir);
            this.registeredTS = true;
            if (this.config.controller) {
                this.config.controller.fork = (modulePath, args, options) => fork_1.fork(this.baseDir, modulePath, args, options);
            }
        }
        _1.addDefaultMiddleware([tracking_1.trackingMiddleware]);
        super.load();
        if (!fs.existsSync(this.metadataPath)) {
            fs.mkdirSync(this.metadataPath);
        }
        // TODO 待分离tracking插件
        if (this.etConfig.useTracking) {
            egg_typed_tracking_1.inspectPromiseAll();
            egg_aop_1.setCreateInstanceHook((inst, _app, ctx) => {
                if (!ctx || !getSymbol(ctx, tracking_1.noTrackingSymbol)) {
                    egg_typed_tracking_1.trackingInstance(inst);
                }
                return inst;
            });
        }
        // load app files
        if (this.etConfig.autoLoadDir) {
            this.autoLoadApp();
        }
        if (this.etConfig.useMetaApi) {
            require('./meta/controller');
        }
        // 组件依赖关系元信息
        fs.writeFileSync(path.join(this.metadataPath, 'ref_map.json'), JSON.stringify(meta_1.getRefMapMetadata(this.app.config), null, 2), { encoding: 'utf8' });
    }
    autoLoadDir(dirPath) {
        fs.readdirSync(dirPath)
            .filter(dir => [].concat(this.etConfig.excludeDir).indexOf(dir) < 0)
            .forEach(dirName => {
            const fullPath = path.join(dirPath, dirName);
            const stat = fs.statSync(fullPath);
            if (stat.isDirectory()) {
                this.autoLoadDir(fullPath);
            }
            else if (stat.isFile()) {
                const ext = path.parse(dirName).ext;
                if (this.etConfig.useTSRuntime) {
                    if (ext === '.ts' && dirName.indexOf('.d.ts') < 0) {
                        loader_util_1.loadFile(fullPath);
                    }
                }
                else {
                    if (ext === '.js') {
                        loader_util_1.loadFile(fullPath);
                    }
                }
            }
        });
    }
    autoLoadApp() {
        this.autoLoadDir(path.join(this.baseDir, 'app'));
    }
    loadFile(filepath, ...inject) {
        if (!this.registeredTS) {
            return super.loadFile(filepath, ...inject);
        }
        const tsFilePath = filepath.replace('.js', '.ts');
        if (!fs.existsSync(tsFilePath)) {
            // fallback
            return super.loadFile(filepath, ...inject);
        }
        let ret = loader_util_1.loadFile(tsFilePath);
        if ('default' in ret) {
            ret = ret.default;
        }
        // function(arg1, args, ...) {}
        if (inject.length === 0)
            inject = [this.app];
        return ret instanceof Function ? ret(...inject) : ret;
    }
}
exports.default = AppWorkerLoader;
