"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const egg_controller_1 = require("egg-controller");
const egg_typed_tracking_1 = require("egg-typed-tracking");
const meta_1 = require("../../meta");
class MetadataController {
    constructor(ctx) {
        this.baseDir = ctx.app.baseDir;
        this.app = ctx.app;
    }
    api() {
        return meta_1.getApiMetadata({ config: this.app.config, showHiddenParam: false });
    }
    refmap() {
        return require(path.join(this.baseDir, 'run', 'ref_map.json'));
    }
    tracking(pageNum = 1, pageSize = 15) {
        const offset = pageSize * (pageNum - 1);
        const dataList = meta_1.getTrackingMetadata();
        return {
            pagination: {
                pageNum,
                pageSize,
                total: dataList.length,
            },
            dataList: dataList.slice(offset, offset + pageSize)
        };
    }
    clearTracking() {
        egg_typed_tracking_1.clearTrackingDatas();
        return true;
    }
}
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/api`,
        name: 'api metadata', description: 'Api接口元数据',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], MetadataController.prototype, "api", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/refmap`,
        name: 'refmap metadata', description: '引用元数据',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], MetadataController.prototype, "refmap", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/trackdata`,
        name: 'trackdata',
        description: '跟踪元数据',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Number, Number]),
    tslib_1.__metadata("design:returntype", void 0)
], MetadataController.prototype, "tracking", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/trackdata/clear`,
        method: 'post',
        name: 'clear-trackdata',
        description: '清空元数据',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], MetadataController.prototype, "clearTracking", null);
exports.MetadataController = MetadataController;
