"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const path = require("path");
const egg_controller_1 = require("egg-controller");
const ui_1 = require("../../test/ui");
const egg_typed_tracking_1 = require("egg-typed-tracking");
const error_1 = require("egg-controller/error");
class UiController {
    constructor(ctx) {
        this.config = ctx.app.config.et.uiTest;
        this.uiRootDir = path.join(ctx.app.baseDir, this.config.basePath);
        this.mkdir(this.uiRootDir);
    }
    getUiTest(uiid) {
        return new ui_1.UiBase(path.join(this.uiRootDir, uiid));
    }
    getUiGroup(uiid, gid) {
        return new ui_1.ReqGroupTestBase(path.join(this.uiRootDir, uiid, gid));
    }
    getUiList() {
        return fs.readdirSync(this.uiRootDir).map(uiid => {
            return this.getUiTest(uiid).info;
        }).filter(info => info);
    }
    getUi(uiid) {
        const uiTest = this.getUiTest(uiid);
        return uiTest.reqGroupTests.map(rgt => (Object.assign({}, rgt.info, { gid: rgt.info.id })));
    }
    addUiTest(uiid, title) {
        const uiTest = this.getUiTest(uiid);
        uiTest.info = {
            uiid, title, reqGroups: []
        };
        uiTest.saveInfo();
        return true;
    }
    addUi(uiid, title, tids) {
        const trackingDatas = egg_typed_tracking_1.getTrackingDatas();
        if (trackingDatas.length <= 0) {
            throw new error_1.NotFoundError('调用跟踪为空!');
        }
        const filterTrackingDatas = trackingDatas.filter(trackingData => tids.indexOf(trackingData.id) >= 0);
        const uiTest = this.getUiTest(uiid);
        const reqGroupTest = filterTrackingDatas.map(trackingData => {
            // 数据修剪
            const header = Object.assign({}, trackingData.extInfo.header);
            delete header['content-length'];
            return {
                title: (trackingData.extInfo.route || {}).name,
                stTime: trackingData.time,
                method: trackingData.extInfo.method,
                url: trackingData.extInfo.originalUrl,
                header,
                body: trackingData.extInfo.body,
                judge: '',
                result: trackingData.extInfo.result,
                mockBlackList: [],
                mockWhiteList: []
            };
        });
        uiTest.saveRegGroupTest(title, reqGroupTest);
        return uiTest;
    }
    delUi(uiid) {
        this.getUiTest(uiid).remove();
    }
    delUiGroup(uiid, gid) {
        this.getUiGroup(uiid, gid).remove();
    }
    mkdir(dirPath, onErr) {
        try {
            if (!fs.existsSync(dirPath)) {
                fs.mkdirSync(dirPath);
            }
        }
        catch (error) {
            onErr && onErr(error);
        }
    }
}
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest`,
        name: '获取 ui 测试列表',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "getUiList", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest/:uiid`,
        name: '获取 ui 测试组列表',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "getUi", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest`,
        method: 'post',
        name: 'add-ui-test',
        description: '添加 UI 测试',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "addUiTest", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest/:uiid`,
        method: 'post',
        name: 'add-ui',
        description: '添加 UI 测试组',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String, Array]),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "addUi", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest/:uiid`,
        method: 'delete',
        name: 'del-ui',
        description: '删除 UI 测试',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "delUi", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/uitest/:uiid/group/:gid`,
        method: 'delete',
        name: 'del-ui-group',
        description: '删除 UI 测试分组',
        noTracking: true,
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", void 0)
], UiController.prototype, "delUiGroup", null);
exports.UiController = UiController;
