"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("power-di/lib/utils");
const egg_typed_tracking_1 = require("egg-typed-tracking");
exports.TrackData = egg_typed_tracking_1.TrackData;
const egg_controller_1 = require("egg-controller");
class Config {
    constructor() {
        this.showHiddenParam = true;
    }
}
exports.Config = Config;
function getApiMetadata({ config, showHiddenParam } = new Config) {
    const ctrls = {};
    egg_controller_1.getRoutes(config).forEach(route => {
        const ctrlName = route.typeGlobalName;
        let ctrl = ctrls[ctrlName];
        if (!ctrl) {
            ctrl = ctrls[ctrlName] = [];
        }
        ctrl.push({
            name: route.name,
            functionName: route.functionName,
            typeGlobalName: route.typeGlobalName,
            description: route.description,
            method: [].concat(route.method),
            url: [].concat(route.url instanceof Function ?
                config ? route.url(config) : undefined
                : route.url),
            paramTypes: (route.paramTypes || [])
                .filter(pt => showHiddenParam || !pt.hidden)
                .map((param) => {
                return {
                    name: param.name,
                    type: param.type && utils_1.getGlobalType(param.type),
                    hidden: param.hidden,
                };
            }),
            returnType: route.returnType && utils_1.getGlobalType(route.returnType),
        });
    });
    return ctrls;
}
exports.getApiMetadata = getApiMetadata;
function getRefMapMetadata(config) {
    return egg_controller_1.getRoutes(config).reduce((prev, current) => {
        return utils_1.getRefMap(current.typeClass, prev);
    }, {});
}
exports.getRefMapMetadata = getRefMapMetadata;
function getTrackingMetadata() {
    return egg_typed_tracking_1.getTrackingDatas().sort((a, b) => b.time - a.time);
}
exports.getTrackingMetadata = getTrackingMetadata;
