import 'colors';
import { ResultType } from '../../tracking';
export declare const INFO_FILE_NAME = "info.json";
export declare const MOCK_DATA_DIR = "mock-data";
/** 测试场景 */
export interface SceneData {
    /** 场景唯一标识，目录名 */
    id: string;
    /** 场景标题 */
    title: string;
    /** 场景描述 */
    description?: string;
    /** 请求 Id[] */
    req: string[];
}
/** 请求序列 */
export interface TestReqData {
    id?: string;
    /** 请求介绍 */
    title?: string;
    /** 开始时间 */
    stTime: number;
    /** 请求方法 */
    method: string;
    /** 请求路径 */
    url: string;
    /** header */
    header: {
        [key: string]: string;
    };
    /** data */
    body: any;
    /** 返回结果判定函数 */
    judge: string;
    /** 请求返回值 */
    result: {
        status?: number;
        body?: any;
        error?: any;
    };
    /** mock数据黑名单，'clsName.method' 格式 */
    mockBlackList: string[];
    /** mock数据白名单，'clsName.method' 格式 */
    mockWhiteList: string[];
    /** mock数据，待分离 */
    mockData?: ReqMockDataType;
}
/** 请求 Mock 数据类型 */
export interface ReqMockDataType {
    /** 类名 */
    [clsName: string]: {
        /** 方法名 */
        [method: string]: MockDataType[];
    };
}
/** Mock 数据类型 */
export interface MockDataType extends ResultType {
    args: any[];
}
export declare class TestBase {
    protected dir: string;
    protected readonly infoPath: string;
    constructor(dir: string);
    protected mkdir(dir: string): void;
    protected readFile(filePath: string): any;
    protected writeFile(filePath: string, data: any): void;
    protected saveInfo(data: any): void;
    protected loadInfo(): any;
    remove(): void;
}
export declare type ReqRunMode = 
/** 正常Mock */
'normal' | 
/** 尝试全量更新Mock */
'update' | 
/** 尝试只修复错误Mock */
'fix';
export declare class ReqTestBase extends TestBase {
    info: TestReqData;
    protected readonly mockDir: string;
    protected mode: ReqRunMode;
    private startMockTime;
    private tmpMockData;
    constructor(dir: string);
    setMode(mode: ReqRunMode): this;
    loadMockData(): void;
    saveMockData(): void;
    startMockData(): void;
    stopMockData(): void;
    saveInfo(): void;
    addMockData(clsName: string, methodName: string, data: MockDataType): void;
    private mockFunc;
}
export declare class SceneTestBase<ReqTestType extends ReqTestBase = ReqTestBase> extends TestBase {
    info: SceneData;
    readonly testReq: ReqTestType[];
    protected _testReq: ReqTestType[];
    constructor(dir: string);
    setReqRunMode(mode: ReqRunMode): this;
    protected loadReqTest(): void;
    saveInfo(): void;
    createReqTest(info: TestReqData): ReqTestBase;
}
