"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const util_1 = require("egg-controller/lib/transformer/util");
console.log('[egg-typed] load transformer: orm model.');
function transformer(_) {
    const typeChecker = _.getTypeChecker();
    return {
        before(ctx) {
            return (sourceFile) => {
                // 遍历文件中所有的 class
                sourceFile.statements
                    .filter(n => ts.isClassDeclaration(n) && util_1.isDecoratorNameInclude(n, 'model'))
                    .forEach((node) => {
                    const decorator = node.decorators.find(dec => {
                        return dec.expression.expression.escapedText === 'model';
                    });
                    let expression = decorator.expression;
                    if (!expression.arguments.length) {
                        expression = decorator.expression = ts.updateCall(expression, expression.expression, undefined, [
                            ts.createObjectLiteral([], false)
                        ]);
                    }
                    // @model 配置参数
                    const config = expression.arguments[0];
                    if (!ts.isObjectLiteralExpression(config)) {
                        console.warn('[egg-typed]orm transformer:', `@model of class [${node.name.escapedText}] param is not a ObjectLiteral.`);
                        return;
                    }
                    let columnField = getDecoratorField(config, 'columns');
                    let referenceField = getDecoratorField(config, 'references');
                    node.members.forEach(member => {
                        if (!ts.isPropertyDeclaration(member)
                            || !ts.isIdentifier(member.name)
                            || ts.isFunctionLike(member.type)) {
                            return;
                        }
                        const typeIdentifier = getTypeIdentifier(member.type, sourceFile, typeChecker);
                        if (util_1.isDecoratorNameInclude(member, 'reference') ||
                            (typeIdentifier.typeRef && !util_1.isDecoratorNameInclude(member, 'column'))) {
                            referenceField.properties = ts.createNodeArray([
                                ...referenceField.properties,
                                ts.createPropertyAssignment(`${member.name.escapedText}`, ts.createObjectLiteral([
                                    ts.createPropertyAssignment('model', typeIdentifier.identifier),
                                    typeIdentifier.array && ts.createPropertyAssignment('toMany', ts.createTrue()),
                                    typeIdentifier.promise && ts.createPropertyAssignment('lazy', ts.createTrue()),
                                ].filter(p => p)))
                            ]);
                        }
                        else {
                            columnField.properties = ts.createNodeArray([
                                ...columnField.properties,
                                ts.createPropertyAssignment(`${member.name.escapedText}`, ts.createObjectLiteral([
                                    ts.createPropertyAssignment('type', typeIdentifier.identifier),
                                ]))
                            ]);
                        }
                    });
                });
                return util_1.walker(sourceFile, ctx);
            };
        },
    };
}
exports.default = transformer;
function getDecoratorField(config, fieldName) {
    let field;
    field = config.properties.find(p => {
        return ts.isIdentifier(p.name) && p.name.escapedText === fieldName;
    });
    if (!field) {
        field = ts.createObjectLiteral();
        config.properties = ts.createNodeArray([
            ...config.properties,
            ts.createPropertyAssignment(fieldName, field),
        ]);
    }
    else {
        field = field.initializer;
    }
    return field;
}
function getTypeIdentifier(typeNode, sourceFile, typeChecker) {
    const type = typeChecker.getTypeFromTypeNode(typeNode);
    if (type.flags & ts.TypeFlags.String ||
        type.flags & ts.TypeFlags.StringLike ||
        type.flags & ts.TypeFlags.Enum ||
        type.flags & ts.TypeFlags.EnumLike) {
        return { identifier: ts.createIdentifier('String') };
    }
    else if (type.flags & ts.TypeFlags.Number ||
        type.flags & ts.TypeFlags.NumberLike) {
        return { identifier: ts.createIdentifier('Number') };
    }
    else if (type.flags & ts.TypeFlags.Boolean ||
        type.flags & ts.TypeFlags.BooleanLike) {
        return { identifier: ts.createIdentifier('Boolean') };
    }
    else if (type.flags & ts.TypeFlags.Any) {
        return { identifier: ts.createIdentifier('Object') };
    }
    else if (type.flags & ts.TypeFlags.Object) {
        const objType = type;
        if (objType.objectFlags & ts.ObjectFlags.Interface ||
            objType.objectFlags & ts.ObjectFlags.Anonymous) {
            return { identifier: ts.createIdentifier('Object') };
        }
        switch (typeNode.kind) {
            case ts.SyntaxKind.ArrayType:
                const arrayNode = typeNode;
                return Object.assign({}, getTypeIdentifier(arrayNode.elementType, sourceFile, typeChecker), { array: true });
            case ts.SyntaxKind.TypeLiteral:
                // mapper json
                return { identifier: ts.createIdentifier('Object') };
            case ts.SyntaxKind.TypeReference:
                const typeRefNode = typeNode;
                const result = {
                    identifier: typeRefNode.typeName,
                    typeRef: true,
                };
                switch (typeRefNode.typeName.escapedText) {
                    case 'Array':
                        return Object.assign({}, result, getTypeIdentifier(typeRefNode.typeArguments[0], sourceFile, typeChecker), { array: true });
                    case 'Promise':
                        return Object.assign({}, result, getTypeIdentifier(typeRefNode.typeArguments[0], sourceFile, typeChecker), { promise: true });
                }
                sourceFile.statements
                    .filter(n => ts.isImportDeclaration(n))
                    .forEach((im) => {
                    const nb = im.importClause.namedBindings;
                    const el = nb.elements.find(el => {
                        return el.name.escapedText === typeRefNode.typeName.escapedText;
                    });
                    if (el) {
                        result.identifier = el.name;
                        im.flags = im.flags | ts.NodeFlags.Synthesized;
                    }
                });
                return result;
        }
    }
    return { identifier: ts.createIdentifier('Object') };
}
