"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const egg_aop_1 = require("egg-aop");
const utils_1 = require("power-di/utils");
tslib_1.__exportStar(require("orm-ts"), exports);
function getRepositoryKey(modelType) {
    return `${utils_1.getGlobalType(modelType)}@Repository`;
}
function registerRepository(modelType, repoType) {
    egg_aop_1.register('Context', repoType, getRepositoryKey(modelType))(repoType);
}
function repository(modelType) {
    return (target) => {
        registerRepository(modelType, target);
    };
}
exports.repository = repository;
//#region 添加MySQL提供程序
const co = require("co");
const egg_aop_2 = require("egg-aop");
const orm_ts_1 = require("orm-ts");
let BaseRepositoryType = orm_ts_1.BaseRepository;
function setBaseRepository(target) {
    BaseRepositoryType = target;
}
exports.setBaseRepository = setBaseRepository;
let MySQLProvider = class MySQLProvider {
    constructor(ctx) {
        this.ctx = ctx;
    }
    getRepositoryByModelClass(modelType) {
        if (!modelType) {
            throw new Error('No modelType!');
        }
        const ormConfig = this.ctx.app.config.orm;
        const key = getRepositoryKey(modelType);
        if (!egg_aop_1.typeLoader.has(key)) {
            class TmpRepository extends BaseRepositoryType {
                constructor() {
                    super(modelType, ormConfig);
                }
            }
            registerRepository(modelType, TmpRepository);
            return this.getRepositoryByModelClass(modelType);
        }
        return egg_aop_2.getInstance(key, this.ctx.app, this.ctx);
    }
    get mysql() {
        return this.ctx._transactionConnection || this.ctx.app.mysql;
    }
    queryOne(sql, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield co(this.mysql.queryOne(sql, params));
        });
    }
    query(sql, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield co(this.mysql.query(sql, params));
        });
    }
};
MySQLProvider = tslib_1.__decorate([
    egg_aop_1.context(orm_ts_1.IProvider),
    tslib_1.__metadata("design:paramtypes", [Object])
], MySQLProvider);
exports.MySQLProvider = MySQLProvider;
//#endregion
