"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const path = require("path");
const utils_1 = require("power-di/utils");
const egg_aop_1 = require("egg-aop");
const bootstrap_1 = require("egg-mock/bootstrap");
const base_1 = require("./lib/test/base");
const di_1 = require("./di");
const orm_1 = require("./orm");
/**
 * getAllTestScene
 * @param testSceneDir
 * @param whiteList scene id list
 */
function getAllTestScene(testSceneDir, whiteList = []) {
    return fs
        .readdirSync(path.join(testSceneDir))
        .filter(dir => fs.statSync(path.join(testSceneDir, dir)).isDirectory())
        .map(dir => {
        return new SceneTest(path.join(testSceneDir, dir));
    })
        .filter(info => info && (!whiteList.length || whiteList.find(s => s === info.info.id)));
}
exports.getAllTestScene = getAllTestScene;
class ReqTest extends base_1.ReqTestBase {
    runTest(context) {
        const { info } = this;
        it(`${info.title ? `${info.title}` : ''}[${info.method} ${info.url}][${info.id}]`, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.startMockData();
            return new Promise((resolve, reject) => {
                const request = bootstrap_1.app.httpRequest()[info.method.toLowerCase()](info.url);
                Object.keys(info.header || {}).forEach(key => {
                    request.set(key, info.header[key]);
                });
                if (info.body && Object.keys(info.body).length) {
                    request.send(info.body);
                }
                request
                    .then((res, err) => {
                    this.stopMockData();
                    if (err) {
                        console.error(err);
                        reject(err);
                    }
                    if (info.judge) {
                        const ret = new Function('with(this){ ' + info.judge + '}').call(context);
                        if (ret === false) {
                            throw new Error('判定失败');
                        }
                    }
                    if (info.result) {
                        if (info.result.status) {
                            bootstrap_1.assert.equal(res.status, info.result.status, `status, ${JSON.stringify(res.text)}`);
                        }
                        if (info.result.body) {
                            const accept = info.header['accept'] || '';
                            let resBody = res.status === 302 || accept.includes('text/html') ? res.text : res.body;
                            bootstrap_1.assert.deepEqual(resBody, info.result.body, `body, ${JSON.stringify(res.text)}`);
                        }
                        if (info.result.error) {
                            bootstrap_1.assert.deepEqual({
                                code: res.body.code,
                                message: res.body.message,
                            }, info.result.error, 'error');
                        }
                    }
                    resolve();
                })
                    .catch((err) => {
                    this.stopMockData();
                    console.error(err);
                    reject(err);
                });
            });
        }));
    }
}
exports.ReqTest = ReqTest;
class SceneTest extends base_1.SceneTestBase {
    loadReqTest() {
        this._testReq = this.info.req.map(reqId => {
            return new ReqTest(path.join(this.dir, reqId));
        });
    }
    /**
     * run req test
     * @param reqWhiteList req id list
     */
    runTest(reqWhiteList = []) {
        const context = {};
        describe(`[SceneTest:${this.info.id}][${this.info.title}]`, () => {
            this.testReq
                .filter(req => !reqWhiteList.length || reqWhiteList.find(rid => req.info.id === rid))
                .map(r => r.runTest(context));
        });
    }
}
exports.SceneTest = SceneTest;
class GlobalMock {
    constructor() {
        this.mockData = new Map();
    }
    start() {
        egg_aop_1.setCreateInstanceHook(this.hook);
    }
    mock(type, mockFunc) {
        this.mockData.set(utils_1.getGlobalType(type), mockFunc);
    }
    stop() {
        egg_aop_1.removeCreateInstanceHook(this.hook);
    }
    hook(inst) {
        const mockFunc = this.mockData.get(utils_1.getGlobalType(inst.constructor));
        if (mockFunc) {
            mockFunc(inst);
        }
        return inst;
    }
}
exports.GlobalMock = GlobalMock;
class TestSuite {
    constructor(_app = bootstrap_1.app) {
        this.globalMock = new GlobalMock();
        this.app = _app;
        this.ctx = _app.mockContext();
    }
    get(clsType) {
        return di_1.getInstance(clsType, this.ctx.app, this.ctx);
    }
    mock(type, func) {
        func(this.get(type));
    }
    getRepo(modelType) {
        const provider = this.get(orm_1.IProvider);
        return provider.getRepositoryByModelClass(modelType);
    }
    mockRepo(modelType, func) {
        func(this.getRepo(modelType));
    }
}
exports.TestSuite = TestSuite;
function getTestSuite() {
    return new TestSuite();
}
exports.getTestSuite = getTestSuite;
/** 直接 mock class proto */
function mockProto(proto, method, mockData) {
    bootstrap_1.mock(proto, method, mockData);
}
exports.mockProto = mockProto;
